package sun.security.provider.certpath.ssl;

import java.security.cert.CertStoreParameters;
import java.security.Provider;
import java.util.Arrays;
import javax.net.ssl.SSLEngine;
import java.net.Socket;
import java.security.cert.CertificateException;
import javax.net.ssl.X509ExtendedTrustManager;
import java.security.GeneralSecurityException;
import javax.net.ssl.TrustManager;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSession;
import java.security.cert.CertStore;
import java.security.cert.X509CRL;
import java.security.cert.CRLSelector;
import java.util.Iterator;
import java.util.ArrayList;
import java.util.List;
import java.net.URLConnection;
import java.util.Collections;
import java.security.cert.CertStoreException;
import java.io.IOException;
import javax.net.ssl.HttpsURLConnection;
import java.security.cert.X509Certificate;
import java.util.Collection;
import java.security.cert.CertSelector;
import java.security.InvalidAlgorithmParameterException;
import javax.net.ssl.HostnameVerifier;
import javax.net.ssl.SSLSocketFactory;
import java.net.URI;
import java.security.cert.CertStoreSpi;

public final class SSLServerCertStore extends CertStoreSpi
{
    private final URI uri;
    private static final SSLServerCertStore.GetChainTrustManager trustManager;
    private static final SSLSocketFactory socketFactory;
    private static final HostnameVerifier hostnameVerifier;
    
    SSLServerCertStore(URI uri) throws InvalidAlgorithmParameterException {
        super(null);
        this.uri = uri;
    }
    
    @Override
    public Collection<X509Certificate> engineGetCertificates(CertSelector certSelector) throws CertStoreException {
        try {
            URLConnection openConnection = this.uri.toURL().openConnection();
            if ((openConnection instanceof HttpsURLConnection)) {
                if (SSLServerCertStore.socketFactory == null) {
                    throw new CertStoreException("No initialized SSLSocketFactory");
                }
                HttpsURLConnection httpsURLConnection = (HttpsURLConnection)openConnection;
                httpsURLConnection.setSSLSocketFactory(SSLServerCertStore.socketFactory);
                httpsURLConnection.setHostnameVerifier(SSLServerCertStore.hostnameVerifier);
                synchronized (SSLServerCertStore.trustManager) {
                    try {
                        httpsURLConnection.connect();
                        List<X509Certificate> matchingCerts = getMatchingCerts(SSLServerCertStore.trustManager.serverChain, certSelector);
                        SSLServerCertStore.trustManager.cleanup();
                        return matchingCerts;
                    }
                    catch (IOException ex) {
                        if (SSLServerCertStore.trustManager.exchangedServerCerts) {
                            List<X509Certificate> matchingCerts2 = getMatchingCerts(SSLServerCertStore.trustManager.serverChain, certSelector);
                            SSLServerCertStore.trustManager.cleanup();
                            return matchingCerts2;
                        }
                        throw ex;
                    }
                    finally {
                        SSLServerCertStore.trustManager.cleanup();
                    }
                }
            }
        }
        catch (IOException cause) {
            throw new CertStoreException(cause);
        }
        return Collections.emptySet();
    }
    
    private static List<X509Certificate> getMatchingCerts(List<X509Certificate> list, CertSelector certSelector) {
        if (certSelector == null) {
            return list;
        }
        ArrayList list2 = new ArrayList(list.size());
        for (X509Certificate x509Certificate : list) {
            if (certSelector.match(x509Certificate)) {
                list2.add(x509Certificate);
            }
            else {
                continue;
            }
        }
        return list2;
    }
    
    @Override
    public Collection<X509CRL> engineGetCRLs(CRLSelector crlSelector) throws CertStoreException {
        throw new UnsupportedOperationException();
    }
    
    static CertStore getInstance(URI uri) throws InvalidAlgorithmParameterException {
        return new SSLServerCertStore.CS(new SSLServerCertStore(uri), null, "SSLServer", null);
    }
    
    static {
        trustManager = new SSLServerCertStore.GetChainTrustManager();
        hostnameVerifier = new HostnameVerifier() {
            @Override
            public boolean verify(String s, SSLSession sslSession) {
                return true;
            }
        };
        SSLSocketFactory socketFactory2;
        try {
            SSLContext instance = SSLContext.getInstance("SSL");
            instance.init(null, new TrustManager[] { SSLServerCertStore.trustManager }, null);
            socketFactory2 = instance.getSocketFactory();
        }
        catch (GeneralSecurityException ex) {
            socketFactory2 = null;
        }
        socketFactory = socketFactory2;
    }
    
    private static class GetChainTrustManager extends X509ExtendedTrustManager
    {
        private List<X509Certificate> serverChain;
        private boolean exchangedServerCerts;
        
        private GetChainTrustManager() {
            this.serverChain = Collections.emptyList();
            this.exchangedServerCerts = false;
        }
        
        @Override
        public X509Certificate[] getAcceptedIssuers() {
            return new X509Certificate[0];
        }
        
        @Override
        public void checkClientTrusted(X509Certificate[] array, String s) throws CertificateException {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void checkClientTrusted(X509Certificate[] array, String s, Socket socket) throws CertificateException {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void checkClientTrusted(X509Certificate[] array, String s, SSLEngine sslEngine) throws CertificateException {
            throw new UnsupportedOperationException();
        }
        
        @Override
        public void checkServerTrusted(X509Certificate[] a, String s) throws CertificateException {
            this.exchangedServerCerts = true;
            this.serverChain = ((a == null) ? Collections.emptyList() : Arrays.asList(a));
        }
        
        @Override
        public void checkServerTrusted(X509Certificate[] array, String s, Socket socket) throws CertificateException {
            this.checkServerTrusted(array, s);
        }
        
        @Override
        public void checkServerTrusted(X509Certificate[] array, String s, SSLEngine sslEngine) throws CertificateException {
            this.checkServerTrusted(array, s);
        }
        
        void cleanup() {
            this.exchangedServerCerts = false;
            this.serverChain = Collections.emptyList();
        }
    }
    
    private static class CS extends CertStore
    {
        protected CS(CertStoreSpi storeSpi, Provider provider, String type, CertStoreParameters params) {
            super(storeSpi, provider, type, params);
        }
    }
}
